# tests/test_ahs_system.py -- versión 1.0

import pytest
import ast
import json
import sys
from pathlib import Path

# Añadir directorio raíz al path
sys.path.insert(0, str(Path(__file__).parent.parent))

from encoder import AHSEncoderFull, encode_python
from decoder import decode_ahs, decode_python, validate_roundtrip

class TestAHSSystem:
    """Test suite completo para el sistema AHS."""
    
    def setup_method(self):
        """Configuración para cada test."""
        self.encoder = AHSEncoderFull()
    
    def test_empty_file(self):
        """Test caso 1: archivo vacío debe devolver estructura vacía."""
        source = ""
        ahs, mapping = self.encoder.encode(source)
        
        assert ahs == ""
        assert json.loads(mapping) == {}
    
    def test_empty_file_with_whitespace(self):
        """Test archivo con solo espacios/newlines."""
        source = "   \n\n  \n"
        ahs, mapping = self.encoder.encode(source)
        
        assert ahs == ""
        assert json.loads(mapping) == {}
    
    def test_simple_import(self):
        """Test import simple."""
        source = "import math"
        ahs, mapping = self.encoder.encode(source)
        
        symtab = json.loads(mapping)
        assert ahs == "I@0"
        assert "@0" in symtab
        assert symtab["@0"] == "import math"
    
    def test_from_import(self):
        """Test from import."""
        source = "from math import pi"
        ahs, mapping = self.encoder.encode(source)
        
        symtab = json.loads(mapping)
        assert ahs == "I@0"
        assert symtab["@0"] == "from math import pi"
    
    def test_simple_function(self):
        """Test caso 2: función simple debe hacer round-trip."""
        source = """def hello():
    return "world" """
        
        ahs, mapping = self.encoder.encode(source)
        decoded = decode_ahs(ahs, mapping)
        
        assert validate_roundtrip(source, decoded)
        assert "F@0" in ahs
        assert "^" in ahs
    
    def test_function_with_parameters(self):
        """Test función con parámetros y tipo hints."""
        source = """def circle_area(radius: float) -> float:
    return math.pi * radius ** 2"""
        
        ahs, mapping = self.encoder.encode(source)
        decoded = decode_ahs(ahs, mapping)
        
        assert validate_roundtrip(source, decoded)
    
    def test_class_definition(self):
        """Test definición de clase simple."""
        source = """class Calculator:
    def add(self, a, b):
        return a + b"""
        
        ahs, mapping = self.encoder.encode(source)
        decoded = decode_ahs(ahs, mapping)
        
        assert validate_roundtrip(source, decoded)
        assert "C@0" in ahs
        assert "F@1" in ahs
        assert ahs.count("^") == 2  # Una para clase, una para función
    
    def test_nested_classes(self):
        """Test caso 3: clases anidadas y funciones con docstrings."""
        source = '''class Outer:
    """Clase externa con docstring largo.
    
    Esta clase demuestra el manejo de:
    - Clases anidadas
    - Docstrings multilínea
    - Métodos complejos
    """
    
    def method1(self):
        return "outer method"
    
    class Inner:
        """Clase interna."""
        
        def inner_method(self):
            """Método interno con docstring."""
            return "inner method"'''
        
        ahs, mapping = self.encoder.encode(source)
        decoded = decode_ahs(ahs, mapping)
        
        assert validate_roundtrip(source, decoded)
    
    def test_module_level_assignment(self):
        """Test asignaciones a nivel de módulo."""
        source = """PI = 3.14159
import math
EULER = 2.71828"""
        
        ahs, mapping = self.encoder.encode(source)
        decoded = decode_ahs(ahs, mapping)
        
        assert validate_roundtrip(source, decoded)
        assert "a@0" in ahs  # PI assignment
        assert "I@1" in ahs  # import
        assert "a@2" in ahs  # EULER assignment
    
    def test_async_function(self):
        """Test función asíncrona."""
        source = """async def fetch_data():
    await some_operation()
    return data"""
        
        ahs, mapping = self.encoder.encode(source)
        decoded = decode_ahs(ahs, mapping)
        
        assert validate_roundtrip(source, decoded)
        assert "F@0" in ahs
    
    def test_complex_module(self):
        """Test módulo complejo con múltiples elementos."""
        source = '''"""Módulo de ejemplo complejo."""

import os
from typing import Optional, List

VERSION = "1.0.0"

class DataProcessor:
    """Procesador de datos."""
    
    def __init__(self, config: dict):
        self.config = config
    
    def process(self, data: List[str]) -> Optional[str]:
        """Procesa una lista de strings."""
        if not data:
            return None
        return "\\n".join(data)
    
    async def async_process(self, data):
        """Versión asíncrona del procesamiento."""
        result = self.process(data)
        return result

def utility_function():
    """Función utilitaria."""
    return "utility"

DEBUG = True'''
        
        ahs, mapping = self.encoder.encode(source)
        decoded = decode_ahs(ahs, mapping)
        
        assert validate_roundtrip(source, decoded)
    
    def test_malformed_ahs(self):
        """Test manejo de AHS malformado."""
        malformed_ahs = "X@0Y@1"  # Tokens inválidos
        mapping = '{"@0": "import math"}'
        
        with pytest.raises(ValueError):
            decode_ahs(malformed_ahs, mapping)
    
    def test_missing_reference(self):
        """Test referencia faltante en mapping."""
        ahs = "F@0^"
        mapping = '{}'  # Sin @0
        
        with pytest.raises(ValueError):
            decode_ahs(ahs, mapping)
    
    def test_unbalanced_blocks(self):
        """Test bloques mal balanceados."""
        ahs = "F@0"  # Función sin cierre
        mapping = '{"@0": "def test(): pass"}'
        
        # Esto no debería lanzar error al decodificar,
        # pero sí al validar el código resultante
        decoded = decode_ahs(ahs, mapping)
        # El código resultante tendrá problemas de indentación
    
    def test_invalid_json_mapping(self):
        """Test mapping JSON inválido."""
        ahs = "I@0"
        invalid_mapping = '{"@0": invalid json}'
        
        with pytest.raises(ValueError):
            decode_ahs(ahs, invalid_mapping)
    
    def test_encoding_functions(self):
        """Test funciones de conveniencia."""
        source = "import math\ndef add(a, b): return a + b"
        
        # Test encode_python
        result = encode_python(source)
        assert "ahs" in result
        assert "map" in result
        
        # Test decode_python
        decoded = decode_python(result)
        assert validate_roundtrip(source, decoded)
    
    def test_compression_metrics(self):
        """Test que la compresión efectivamente reduce el tamaño para casos típicos."""
        # Código repetitivo que debería comprimirse bien
        source = """import os
import sys
import json

def func1():
    return "result1"

def func2():
    return "result2"

def func3():
    return "result3"

class Class1:
    def method1(self):
        return "method1"
    
    def method2(self):
        return "method2" """
        
        ahs, mapping = self.encoder.encode(source)
        
        original_size = len(source)
        compressed_size = len(ahs) + len(mapping)
        
        # Para código típico, deberíamos ver alguna compresión
        # aunque AHS no es principalmente un algoritmo de compresión
        print(f"Original: {original_size}, Compressed: {compressed_size}")
        print(f"AHS: {ahs}")
        print(f"Mapping length: {len(mapping)}")

# Tests de integración
class TestIntegration:
    """Tests de integración del sistema completo."""
    
    def test_python_examples(self):
        """Test con ejemplos de código Python real."""
        examples = [
            # Ejemplo de utils.py típico
            '''import math
from typing import List, Optional

def calculate_circle_area(radius: float) -> float:
    """Calcula el área de un círculo."""
    return math.pi * radius ** 2

def fibonacci(n: int) -> List[int]:
    """Genera secuencia de Fibonacci."""
    if n <= 0:
        return []
    elif n == 1:
        return [0]
    elif n == 2:
        return [0, 1]
    
    fib = [0, 1]
    for i in range(2, n):
        fib.append(fib[i-1] + fib[i-2])
    return fib

class MathUtils:
    """Utilidades matemáticas."""
    
    @staticmethod
    def is_prime(n: int) -> bool:
        """Verifica si un número es primo."""
        if n < 2:
            return False
        for i in range(2, int(math.sqrt(n)) + 1):
            if n % i == 0:
                return False
        return True
    
    @classmethod
    def gcd(cls, a: int, b: int) -> int:
        """Calcula el máximo común divisor."""
        while b:
            a, b = b, a % b
        return a''',
        ]
        
        encoder = AHSEncoderFull()
        
        for i, source in enumerate(examples):
            ahs, mapping = encoder.encode(source)
            decoded = decode_ahs(ahs, mapping)
            
            assert validate_roundtrip(source, decoded), f"Falló ejemplo {i+1}"
    
    def test_edge_cases(self):
        """Test casos límite."""
        edge_cases = [
            # Solo comentarios y docstrings (sin código ejecutable)
            '"""Módulo solo con docstring."""',
            
            # Función lambda (no debería ser capturada como FunctionDef)
            'square = lambda x: x ** 2',
            
            # Decoradores
            '''@property
def value(self):
    return self._value''',
            
            # Múltiples imports en líneas separadas
            '''import os
import sys
import json
import math''',
        ]
        
        encoder = AHSEncoderFull()
        
        for i, source in enumerate(edge_cases):
            try:
                ahs, mapping = encoder.encode(source)
                decoded = decode_ahs(ahs, mapping)
                
                # Para casos válidos, debe hacer round-trip
                if source.strip():  # Si no es vacío
                    assert validate_roundtrip(source, decoded), f"Falló edge case {i+1}"
            except Exception as e:
                pytest.fail(f"Edge case {i+1} falló con error: {e}")

# Fixtures para casos de prueba específicos
@pytest.fixture
def sample_utils_module():
    """Fixture con un módulo utils.py típico."""
    return '''"""Utilidades matemáticas y de string."""

import math
import re
from typing import List, Optional, Union

# Constantes
PI = 3.14159265359
EULER_NUMBER = 2.71828182846

def calculate_area(radius: float) -> float:
    """Calcula el área de un círculo."""
    return PI * radius ** 2

def validate_email(email: str) -> bool:
    """Valida formato de email usando regex."""
    pattern = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\\.[a-zA-Z]{2,}
    return bool(re.match(pattern, email))

class StringProcessor:
    """Procesador de strings con múltiples funcionalidades."""
    
    def __init__(self, case_sensitive: bool = True):
        self.case_sensitive = case_sensitive
    
    def clean_text(self, text: str) -> str:
        """Limpia texto removiendo espacios extra."""
        return ' '.join(text.split())
    
    def count_words(self, text: str) -> int:
        """Cuenta palabras en un texto."""
        return len(self.clean_text(text).split())
    
    @staticmethod
    def reverse_string(s: str) -> str:
        """Invierte un string."""
        return s[::-1]

async def fetch_and_process(url: str) -> Optional[str]:
    """Función asíncrona ejemplo."""
    # Simulación de fetch
    await asyncio.sleep(0.1)
    return f"Processed: {url}"

# Variable global
DEBUG_MODE = False'''

def test_with_sample_utils(sample_utils_module):
    """Test usando el módulo utils de ejemplo."""
    encoder = AHSEncoderFull()
    
    ahs, mapping = encoder.encode(sample_utils_module)
    decoded = decode_ahs(ahs, mapping)
    
    # Verificar round-trip
    assert validate_roundtrip(sample_utils_module, decoded)
    
    # Verificar que contiene los elementos esperados
    assert "I@" in ahs  # imports
    assert "a@" in ahs  # assignments (PI, EULER_NUMBER, DEBUG_MODE)
    assert "F@" in ahs  # functions
    assert "C@" in ahs  # class StringProcessor
    
    # Verificar estructura del mapping
    symtab = json.loads(mapping)
    assert len(symtab) > 0
    
    # Cada referencia debe estar presente
    import re
    refs = re.findall(r'@\d+', ahs)
    for ref in refs:
        assert ref in symtab